@@ -1,11 +1,14 @@
 /* App.h
  * Application class of the Cenon project
  *
- * Copyright (C) 1995-2007 by vhf interservice GmbH
+ * Copyright (C) 1995-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1995-08-10
- * modified: 2007-07-19
+ * modified: 2011-03-06 (TOOL2D_WEB and TOOL2D_Mark exchanged)
+ *           2010-05-19 (-showWebPage:)
+ *           2010-01-12 (-takeSnapshot:/-restoreSnapshot:)
+ *           2008-07-30 (-projectSettingsPanel)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -31,7 +34,7 @@
 
 #include <AppKit/AppKit.h>
 #include <VHFShared/types.h>
-#include "MyPageLayout.h"
+//#include "MyPageLayout.h"   // FIXME
 #include "Document.h"
 
 /* 2D tools */
@@ -39,19 +42,19 @@
 #define TOOL2D_ROTATE    1
 #define TOOL2D_SCISSOR   2
 #define TOOL2D_ADDPOINT  3
-#define TOOL2D_SINKING   4
-#define TOOL2D_THREAD    5
-#define TOOL2D_MARK      6
+#define TOOL2D_SINKING   4  // CAM
+#define TOOL2D_THREAD    5  // CAM
+#define TOOL2D_WEB       6  // CAM
 #define TOOL2D_LINE      7
 #define TOOL2D_POLYLINE  8
 #define TOOL2D_ARC       9
 #define TOOL2D_RECT     10
 #define TOOL2D_CURVE    11
 #define TOOL2D_TEXT     12
-#define TOOL2D_WEB      13
+#define TOOL2D_MARK     13
 #define TOOL2D_PATH     14
 
-/* Menu tags (used to add menu items) */
+/* Menu tags / index+1 (used to add menu items) */
 #define	MENU_INFO        1
 #define	MENU_DOCUMENT    2
 #define	MENU_EDIT        3
@@ -65,37 +68,40 @@
 #define	MENU_QUIT       10
 
 #define InfoPanelWillDisplay	@"InfoPanelWillDisplay"
+#define ToolPanelWillDisplay	@"ToolPanelWillDisplay"
 
 @interface App : NSApplication
 {
     /* info panel */
     NSPanel	*infoPanel;
-    id		infoVersionNo;		// version number "3.8.3"
-    id		kindOfVersion;		// "Free Software", "Demo", "CAM"
-    id		serialNumber;		// "020001"
+    id		infoVersionNo;          // version number "3.9.0"
+    id		kindOfVersion;          // "Free Software", "Demo", "Licensed"
+    id		serialNumber;           // "020001"
 
     id		helpPanel;
     id		inspectorPanel;
     id		transformPanel;
     id		preferencesPanel;
     id		projectSettingsPanel;	// the project settings panel
-    id		toolPanel;			// the tool panel
+    id		toolPanel;              // the tool panel
     id		tilePanel;
     id		gridPanel;
     id		workingAreaPanel;
     id		intersectionPanel;
 
-    Document	*document;		// last opened document
-    Document	*fixedDocument;		// used from -setCurrentDocument
-    NSMutableArray	*modules;	// list of our loaded modules
-    int		current2DTool;		// the tag of the current 2D tool
+    Document        *document;          // last opened document
+    int             activeWindowNum;    // active document window set by -setActiveDocWindow:
+    Document        *fixedDocument;     // used from -setCurrentDocument
+    NSMutableArray	*modules;           // list of our loaded modules
+    int             current2DTool;      // the tag of the current 2D tool
 
-    id		openPanelAccessory;	// the OpenPanel PS/HPGL/Gerber view
-    id		opaMatrix;
-    id		savePanelAccessory;
-    id		spaFormatPopUp;
-    id		printPanelAccessory;
-    id		ppaRadio;
+    /* accessories (Open-Panel, Save-Panel, Print-Panel) */
+    id          openPanelAccessory; // the Open Panel Accessory PS/HPGL/Gerber view
+    id          opaMatrix;
+    id          savePanelAccessory; // Save Panel Accessory
+    id          spaFormatPopUp;
+    id          printPanelAccessory;
+    id          ppaRadio;
 
     int		appIsRunning;
     BOOL	haveOpenedDocument;	// whether we have opened a document
@@ -106,11 +112,11 @@
     BOOL	shift;
 
     /* import accessories */
-    id	importASCIIAccessory;
-    id	iaaRadio;
-    id	iaaPopup;
-    id	importAccessory;
-    id	iaPopup;
+    id          importASCIIAccessory;   // deprecated, ready to be removed
+    id          iaaRadio;               // deprecated, ready to be removed
+    id          iaaPopup;               // deprecated, ready to be removed
+    id          importAccessory;
+    id          iaPopup;
 
     /* contour panel */
     int		contourUnit;
@@ -128,6 +134,7 @@ void getAppDirectory(char *appDirectory);
 - (NSArray*)modules;
 - (NSString *)currentDirectory;
 - (void)setCurrentDocument:(Document*)docu;
+- (void)setActiveDocWindow:(DocWindow*)win;
 - (Document*)currentDocument;	// document from main window
 - (Document*)openedDocument;	// last opened document
 - (Document*)documentInWindow:(NSWindow*)window;
@@ -139,19 +146,30 @@ void getAppDirectory(char *appDirectory);
 - (NSView*)printPanelAccessory;
 - (id)ppaRadio;
 
+/* Snapshots */
+- (void)takeSnapshot:sender;
+- (void)restoreSnapshot:sender;
+
 /* info panel */
 - (void)displayInfo;
 - (void)showInfo:sender;
+- (NSString*)version;       // 3.9.2
+- (NSString*)compileDate;   // 2010-06-28
 - (id)infoVersionText;
 - (id)infoSerialText;
 
+- (void)checkForUpdate:sender;
+
 - (void)showPrefsPanel:sender;
 - (id)preferencesPanel;
+- (void)showWebPage:(id)sender;
 //- (void)showHelp:sender;
 - (void)showInspectorPanel:sender;
 - (id)inspectorPanel;
 - (void)showTransformPanel:sender;
+- (void)showVectorizer:sender;
 - (void)showProjectSettingsPanel:sender;
+- (id)projectSettingsPanel;
 - (void)showTilePanel:sender;
 - (id)tilePanel;
 - (void)runGridPanel:sender;
@@ -168,8 +186,9 @@ void getAppDirectory(char *appDirectory);
 - (NSArray*)listFromPSFile:(NSString*)fileName;
 - (BOOL)openFile:(NSString *)fileName;
 - (void)import:sender;
-- (void)importASCII:sender;
+//- (void)importASCII:sender;   // moved to CAM module
 - (void)openDocument:sender;
+- (void)changeOpenLocation:(id)sender;
 - (void)save:sender;
 - (void)saveAs:sender;
 - (void)changeSaveType:sender;
@@ -177,6 +196,8 @@ void getAppDirectory(char *appDirectory);
 
 - (int)application:app openFile:(NSString *)path;
 
+/* Tool Panel */
+- (NSPanel*)toolPanel;                  // return tool panel
 - (void)showToolPanel:sender;
 - (void)displayToolPanel:(BOOL)flag;
 - (void)setCurrent2DTool:sender;
