@@ -1,11 +1,15 @@
 /* VGraphic.m
  * Graphic object - root class
  *
- * Copyright (C) 1996-2006 by vhf interservice GmbH
+ * Copyright (C) 1996-2011 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-29
- * modified: 2006-02-16
+ * modified: 2011-05-28 (excluded, -isExcluded, -setExcluded:, +initialize: version=5)
+ *           2011-04-06 (-drawStartAtScale: added)
+ *           2011-03-04 (-size, -setSize: default implementation to make it work for all objects)
+ *           2010-07-28 (-setLabel:, -label added, version = 4)
+ *           2010-07-08 (-transform: added)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -31,48 +35,48 @@
 #include "../DocWindow.h"
 #include "../Document.h"
 #include "../DocView.h"	// scaleFactor, mustDrawPale
-//#include "../OutputStreamDrawing.h"
 #include "VGraphic.h"
 
 @implementation VGraphic
 
-/*
- * The fastKnobFill optimization just keeps a list of black and dark gray
+/* The fastKnobFill optimization just keeps a list of black and dark gray
  * rectangles (the knobbies are made out of black and dark gray rectangles)
  * and emits them in a single NSRectFillList() which is much faster than
  * doing individual rectfills (we also save the repeated setgrays).
  */
-
-static NSRect *blackRectList = NULL;
-static int blackRectSize = 0;
-static int blackRectCount = 0;
+static NSRect *lgrayRectList  = NULL;
+static int lgrayRectSize   = 0;
+static int lgrayRectCount  = 0;
+static NSRect *blackRectList  = NULL;
+static int blackRectSize   = 0;
+static int blackRectCount  = 0;
 static NSRect *dkgrayRectList = NULL;
-static int dkgrayRectSize = 0;
+static int dkgrayRectSize  = 0;
 static int dkgrayRectCount = 0;
 
 
 + (float)maxKnobSizeWithScale:(float)scaleFactor
 {
-    return (KNOBSIZE+2.0) / scaleFactor;
+    return (KNOBSIZE+4.0) / scaleFactor;    // 2008-02-25: was 2.0
 }
 
-+ fastKnobFill:(NSRect)aRect isBlack:(BOOL)isBlack
++ fastKnobFill:(NSRect)aRect isBlack:(int)isBlack
 {
-    if (isBlack)
+    if ( isBlack == YES )
     {
-	if (!blackRectList)
+        if (!blackRectList)
         {   blackRectSize = 16;
-	    blackRectList = NSZoneMalloc((NSZone *)[(NSObject *)NSApp zone], (blackRectSize) * sizeof(NSRect));
-	}
+            blackRectList = NSZoneMalloc((NSZone *)[(NSObject *)NSApp zone], (blackRectSize) * sizeof(NSRect));
+        }
         else if ( blackRectCount >= blackRectSize )
         {
-	    while (blackRectCount >= blackRectSize)
-                blackRectSize <<= 1;
-	    blackRectList = NSZoneRealloc((NSZone *)[(NSObject *)NSApp zone], blackRectList, (blackRectSize) * sizeof(NSRect));
-	}
-	blackRectList[blackRectCount++] = aRect;
+            while (blackRectCount >= blackRectSize)
+                    blackRectSize <<= 1;
+            blackRectList = NSZoneRealloc((NSZone *)[(NSObject *)NSApp zone], blackRectList, (blackRectSize) * sizeof(NSRect));
+        }
+        blackRectList[blackRectCount++] = aRect;
     }
-    else
+    else if ( isBlack == NO )
     {
         if (!dkgrayRectList)
         {   dkgrayRectSize = 16;
@@ -86,22 +90,39 @@ static int dkgrayRectCount = 0;
         }
         dkgrayRectList[dkgrayRectCount++] = aRect;
     }
+    else // 2
+    {
+        if (!lgrayRectList)
+        {   lgrayRectSize = 16;
+            lgrayRectList = NSZoneMalloc((NSZone *)[(NSObject *)NSApp zone], (lgrayRectSize) * sizeof(NSRect));
+        }
+        else if ( lgrayRectCount >= lgrayRectSize )
+        {
+            while (lgrayRectCount >= lgrayRectSize)
+                lgrayRectSize <<= 1;
+            lgrayRectList = NSZoneRealloc((NSZone *)[(NSObject *)NSApp zone], lgrayRectList, (lgrayRectSize) * sizeof(NSRect));
+        }
+        lgrayRectList[lgrayRectCount++] = aRect;
+    }
 
     return self;
 }
 
 + (void)showFastKnobFills
 {
+    if (lgrayRectCount)
+    {   [[NSColor whiteColor] set]; // lightGrayColor
+        NSRectFillList(lgrayRectList, lgrayRectCount);
+        lgrayRectCount = 0;
+    }
     if (blackRectCount)
-    {
-        [[NSColor blackColor] set];
-	NSRectFillList(blackRectList, blackRectCount);
+    {   [[NSColor blackColor] set];
+        NSRectFillList(blackRectList, blackRectCount);
         blackRectCount = 0;
     }
     if (dkgrayRectCount)
-    {
-        [[NSColor lightGrayColor] set];
-	NSRectFillList(dkgrayRectList, dkgrayRectCount);
+    {   [[NSColor lightGrayColor] set];
+        NSRectFillList(dkgrayRectList, dkgrayRectCount);
         dkgrayRectCount = 0;
     }
 }
@@ -113,7 +134,33 @@ static int dkgrayRectCount = 0;
 static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scaleFactor )
 {   NSRect	knob;
     float	scale = 1.0 / scaleFactor;
-    float	knobsize = (selectedKnob) ? KNOBSIZE+1.0 : KNOBSIZE;
+    float	knobsize = (selectedKnob) ? KNOBSIZE+3.0 : KNOBSIZE+1;  // 2008-02-25: was: KNOBSIZE+2.0 : KNOBSIZE;
+
+    knob.size.width = knob.size.height = knobsize*scale;
+    knob.origin.x = origin.x - ((knobsize - 1.0) / 2.0)*scale;
+    knob.origin.y = origin.y - ((knobsize - 1.0) / 2.0)*scale;
+    if ( direct )
+        NSRectFill(knob);
+    else
+        [VGraphic fastKnobFill:knob isBlack:2]; // lgrayRectList
+
+    knob.origin.x = origin.x - ((knobsize - 3.0) / 2.0)*scale;
+    knob.origin.y = origin.y - ((knobsize - 3.0) / 2.0)*scale;
+    knob.size.width = knob.size.height = (knobsize-2.0)*scale;
+    if ( direct )
+        NSRectFill(knob);
+    else
+        [VGraphic fastKnobFill:knob isBlack:YES];
+
+}
+/* Draws the control knobs
+ * direct=YES draws the knob directly to the view
+ * direct=NO draws the knob to the rectlist to be shown with showFastRectList
+ */
+static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float scaleFactor )
+{   NSRect	knob;
+    float	scale = 1.0 / scaleFactor;
+    float	knobsize = (selectedKnob) ? KNOBSIZE+2.0 : KNOBSIZE;
 
     knob.size.width = knob.size.height = knobsize*scale;
     knob.origin.x = origin.x - ((knobsize - 1.0) / 2.0)*scale;
@@ -139,7 +186,7 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
  */
 + (void)initialize
 {
-    [VGraphic setVersion:2];
+    [VGraphic setVersion:5];
     return;
 }
 
@@ -174,8 +221,8 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
 }
 
 + (NSArray*)objectsOfClass:(Class)cls inArray:(NSArray*)array
-{   int			i;
-    NSMutableArray	*mArray = nil;
+{   int             i;
+    NSMutableArray  *mArray = nil;
 
     for ( i=[array count]-1; i>=0; i-- )
     {   VGraphic	*g = [array objectAtIndex:i];
@@ -193,9 +240,13 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
 - init
 {
     pmList = [NSMutableArray new];
+    label = nil;
     color = [[NSColor blackColor] retain];
     isDirectionCCW = YES;
-    dirty = YES;	// need calculation of output
+    dirty = YES;	// needs calculation of output
+
+    relief = NO; reliefType = reliefDirection = 0; reliefFlatness = 1.0;
+
     return self;
 }
 
@@ -203,9 +254,9 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
  * overwritten by subclasses
  */
 - copy
-{   NSMutableData	*data = [[NSMutableData alloc] init];
-    NSArchiver		*ts;
-    id			g;
+{   NSMutableData   *data = [[NSMutableData alloc] init];
+    NSArchiver      *ts;
+    id              g;
 
     /* writes the path to a NSData object and reads it back from it */
     ts = [[NSArchiver alloc] initForWritingWithMutableData:data];
@@ -224,7 +275,7 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
 
 - (NSString*)description
 {
-    return [NSString stringWithFormat:@"%@: width = %f", [self title], width];
+    return [NSString stringWithFormat:@"%@: id = '%@', width = %f", [self title], (label) ? label : @"", width];
 }
 
 - (NSString *)title
@@ -239,6 +290,9 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
 - (void)setSelected:(BOOL)flag	{ isSelected = flag; }
 - (int)selectedKnobIndex	{ return -1; }
 
+- (void)setLabel:(NSString*)newLabel    { [label release]; label = [newLabel retain]; }
+- (NSString*)label                      { return label; }
+
 - (void)setColor:(NSColor *)col
 {
     if (color)
@@ -332,14 +386,16 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
 {
 }
 
-- (void)setSize:(NSSize)size
-{
-    NSLog(@"method -setSize: not implemented");
+- (void)setSize:(NSSize)newSize
+{   NSRect	bRect = [self coordBounds];
+
+    [self scale:((bRect.size.width)  ? newSize.width /bRect.size.width  : 1.0)
+               :((bRect.size.height) ? newSize.height/bRect.size.height : 1.0)
+     withCenter:bRect.origin];
 }
 - (NSSize)size
-{
-    NSLog(@"method -size not implemented");
-    return NSMakeSize(0.0, 0.0);
+{   NSRect	bRect = [self coordBounds];
+    return bRect.size;
 }
 
 - (void)setRadius:(float)r
@@ -352,14 +408,11 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
     return 0.0;
 }
 
-- (BOOL)isLocked;
-{
-    return isLocked;
-}
-- (void)setLocked:(BOOL)l
-{
-    isLocked = l;
-}
+- (BOOL)isExcluded              { return isExcluded; }
+- (void)setExcluded:(BOOL)flag  { isExcluded = flag; }
+
+- (BOOL)isLocked                { return isLocked; }
+- (void)setLocked:(BOOL)l       { isLocked = l; }
 
 - (float)angle
 {
@@ -433,6 +486,13 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
     return NO;
 }
 
+- (void)transform:(NSAffineTransform*)matrix
+{   NSSize  size = NSMakeSize(width, width);
+
+    size = [matrix transformSize:size];
+    width = (Abs(size.width) + Abs(size.height)) / 2;
+}
+
 - (void)scale:(float)x :(float)y withCenter:(NSPoint)cp
 {
 	 
@@ -456,7 +516,7 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
 - (void)drawColorPale:(BOOL)drawPale
 {
     if ( drawPale )
-    {   float	h, s, b, a;
+    {   CGFloat h, s, b, a;
 
         [[color colorUsingColorSpaceName:NSDeviceRGBColorSpace] getHue:&h saturation:&s brightness:&b alpha:&a];
         [[NSColor colorWithCalibratedHue:h saturation:s brightness:(b<0.5) ? 0.5 : b alpha:a] set];
@@ -486,21 +546,48 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
  * opaque then there is a possibility that it might obscure knobbies
  * of Graphics underneath it, so we must emit the cached rectfills
  * before drawing this VGraphic.
+ *
+ * modified: 2008-02-25 (VImage added)
  */
 - (void)drawControls:(NSRect)rect direct:(BOOL)direct scaleFactor:(float)scaleFactor
 {
-    [self drawKnobs:rect direct:direct scaleFactor:scaleFactor];
+    // [self drawKnobs:rect direct:direct scaleFactor:scaleFactor];
+    if ( (NSIsEmptyRect(rect) ||
+          !NSIsEmptyRect(NSIntersectionRect(rect, [self extendedBoundsWithScale:scaleFactor]))) )
+    {
+        if ( VHFIsDrawingToScreen() && isSelected )
+        {   int	i;
+
+            if ( ![self isKindOfClass:[VRectangle class]] && ![self isKindOfClass:[VText class]] &&
+                 ![self isKindOfClass:[VImage class]])
+            {
+                for ( i=[self numPoints]-2; i>0; i-- )
+                    drawControl( [self pointWithNum:i], direct, [self selectedKnobIndex]==i, scaleFactor );
+            }
+        }
+    }
 }
+/* draw only the start and end point - VRectangle, VText
+ * modified: 2008-02-25 (==i instead of ==0)
+ */
 - (void)drawKnobs:(NSRect)rect direct:(BOOL)direct scaleFactor:(float)scaleFactor
 {
     if ( (NSIsEmptyRect(rect) ||
           !NSIsEmptyRect(NSIntersectionRect(rect, [self extendedBoundsWithScale:scaleFactor]))) )
     {
-	if ( VHFIsDrawingToScreen() && isSelected )
-        {   int	i;
+        if ( VHFIsDrawingToScreen() && isSelected )
+        {
+            if ( [self isKindOfClass:[VRectangle class]] || [self isKindOfClass:[VText class]] ||
+                 [self isKindOfClass:[VImage class]] )
+            {   int	i;
 
-            for ( i=[self numPoints]-1; i>=0; i-- )
-                drawKnob( [self pointWithNum:i], direct, [self selectedKnobIndex]==i, scaleFactor );
+                for ( i=[self numPoints]-1; i>=0; i-- )
+                    drawKnob([self pointWithNum:i], direct, [self selectedKnobIndex]==i, scaleFactor);
+            }
+            else
+            {   drawKnob([self pointWithNum:0], direct, [self selectedKnobIndex]==0, scaleFactor);
+                drawKnob([self pointWithNum:MAXINT], direct, [self selectedKnobIndex]==[self numPoints]-1, scaleFactor);
+            }
         }
     }
 }
@@ -534,6 +621,34 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
     PSgrestore();
 }
 
+- (void)drawStartAtScale:(float)scaleFactor
+{   NSPoint         p0, p;
+    float           a, scale = 1.0 / scaleFactor;
+    NSBezierPath    *bPath = [NSBezierPath bezierPath];
+
+    p = [self gradientAt:0.0];
+    a = (!p.x) ?  ((p.y>0)?90.0:270.0) : RadToDeg(atan(p.y/p.x));
+    if (p.x < 0)
+        a += 180.0;
+    p0 = [self pointAt:0.0];
+
+    PSgsave();
+    [[NSColor blackColor] set];
+    [bPath setLineWidth:0.0];
+    [bPath moveToPoint:p0];
+    p = vhfPointRotatedAroundCenter(NSMakePoint(p0.x, p0.y - 6*scale), a, p0);
+    [bPath lineToPoint:p];
+    p = vhfPointRotatedAroundCenter(NSMakePoint(p0.x+2*scale, p0.y - 6*scale), a, p0);
+    [bPath lineToPoint:p];
+    p = vhfPointRotatedAroundCenter(NSMakePoint(p0.x+2*scale, p0.y + 6*scale), a, p0);
+    [bPath lineToPoint:p];
+    p = vhfPointRotatedAroundCenter(NSMakePoint(p0.x, p0.y + 6*scale), a, p0);
+    [bPath lineToPoint:p];
+    [bPath lineToPoint:p0];
+    [bPath fill];
+    PSgrestore();
+}
+
 /* exact coordinate bounds of an object
  * in case of curve this gives a slow calculation but more precise bounds
  * may have zero size!
@@ -840,6 +955,15 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
         [aDecoder decodeValuesOfObjCTypes:"f", &width];
     if ( version >= 2 )
         [aDecoder decodeValuesOfObjCTypes:"c", &isLocked];
+    if ( version >= 3 ) // 2008-03-18
+    {   [aDecoder decodeValuesOfObjCTypes:"ciif", &relief, &reliefType, &reliefDirection, &reliefFlatness];
+        if ( !reliefFlatness )
+            reliefFlatness = 1.0;
+    }
+    if ( version >= 4 ) // 2010-07-28
+        label = [[aDecoder decodeObject] retain];
+    if ( version >= 5 ) // 2011-05-28
+        [aDecoder decodeValuesOfObjCTypes:"c", &isExcluded];
 
     return self;
 }
@@ -849,6 +973,9 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
     [aCoder encodeObject:color];
     [aCoder encodeValuesOfObjCTypes:"f", &width];
     [aCoder encodeValuesOfObjCTypes:"c", &isLocked];
+    [aCoder encodeValuesOfObjCTypes:"ciif", &relief, &reliefType, &reliefDirection, &reliefFlatness];
+    [aCoder encodeObject:label];
+    [aCoder encodeValuesOfObjCTypes:"c", &isExcluded];
 }
 
 /* archiving with property list
@@ -857,30 +984,49 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
 {   NSMutableDictionary	*plist = [NSMutableDictionary dictionaryWithCapacity:10];
 
     [plist setObject:NSStringFromClass([self class]) forKey:@"Class"];
-    if (color != [NSColor blackColor])
+    if ( label && [label length] )
+        [plist setObject:label forKey:@"id"];
+    if ( color && color != [NSColor blackColor] )
         [plist setObject:propertyListFromNSColor(color) forKey:@"color"];
-    if (width)
-        [plist setObject:propertyListFromFloat(width) forKey:@"width"];
-    if (isLocked) [plist setObject:@"1" forKey:@"isLocked"];
+    if (width)          [plist setObject:propertyListFromFloat(width) forKey:@"width"];
+    if (isExcluded)     [plist setObject:@"1" forKey:@"isExcluded"];
+    if (isLocked)       [plist setObject:@"1" forKey:@"isLocked"];
     if (isDirectionCCW) [plist setObject:@"1" forKey:@"ccw"];
     //if (!isDirectionCCW) [plist setObject:@"1" forKey:@"cw"];
+
+    /* Relief */
+    if (relief)          [plist setObject:@"YES" forKey:@"relief"];
+    if (reliefType)      [plist setInt:reliefType forKey:@"reliefType"];
+    if (reliefDirection) [plist setInt:reliefDirection forKey:@"reliefDirection"];
+    if (reliefFlatness != 1.0)  [plist setObject:propertyListFromFloat(reliefFlatness) forKey:@"reliefFlatness"];
+
     return plist;
 }
 - (id)initFromPropertyList:(id)plist inDirectory:(NSString *)directory
 {
     pmList = [NSMutableArray new];
+    label = [[plist objectForKey:@"id"] retain];
     if (!(color = colorFromPropertyList([plist objectForKey:@"color"], [self zone])))
         [self setColor:[NSColor blackColor]];
     width = [plist floatForKey:@"width"];
-    isLocked = ([plist objectForKey:@"isLocked"]) ? YES : NO;
+    isExcluded = ([plist objectForKey:@"isExcluded"]) ? YES : NO;
+    isLocked   = ([plist objectForKey:@"isLocked"])   ? YES : NO;
     //isDirectionCCW = ([plist objectForKey:@"cw"]) ? NO : YES;
     isDirectionCCW = ([plist objectForKey:@"ccw"] || [plist objectForKey:@"isDirectionCCW"]) ? YES : NO;
+
+    /* Relief */
+    relief          = (([plist objectForKey:@"relief"]) ? YES : NO);
+    reliefType      = [plist intForKey:@"reliefType"];
+    reliefDirection = [plist intForKey:@"reliefDirection"];
+    if ( !(reliefFlatness = [plist floatForKey:@"reliefFlatness"]) )
+        reliefFlatness = 1.0;
+
     dirty = YES;
     return self;
 }
 
-- (BOOL)isDirty				{ return dirty; }
-- (void)setDirty:(BOOL)flag		{ dirty = flag; }
+- (BOOL)isDirty             { return dirty; }
+- (void)setDirty:(BOOL)flag { dirty = flag; }
 - (void)setOutputStream:(id)stream
 {
     [outputStream release];
@@ -893,6 +1039,7 @@ static void drawKnob( NSPoint origin, BOOL direct, BOOL selectedKnob, float scal
 
 - (void)dealloc
 {
+    [label release];
     [color release];
     [outputStream release];
     [pmList release];
