@@ -1,11 +1,13 @@
 /* VGroup.m
  * Group of graphic objects
  *
- * Copyright (C) 1996-2005 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-29
- * modified: 2005-09-03 (punch stuff)
+ * modified: 2012-07-17 (*pts=NULL, *iPts=NULL initialized)
+ *           2012-04-20 (-coordBounds zero size Bounds are not recognized)
+ *           2010-07-08 (relocated -setList:)
  *
  * the group is implemented as follows:
  * - all objects are stored in a list with absolute coordinates
@@ -125,19 +127,29 @@
 
 - (NSString*)title		{ return @"Group"; }
 
-- list
+- (NSMutableArray*)list
 {
     return list;
 }
+- (void)setList:(NSArray*)aList
+{
+    [list release];
+    list = [aList retain];
+    [self setColorNew];
+    coordBounds = bounds = NSZeroRect;
+    dirty = YES;
+}
 
 - (unsigned)countRecursive
-{   int	i, cnt = [list count];
+{   int	i, cnt = 0;
 
     for (i=[list count]-1; i>=0; i--)
     {	id	g = [list objectAtIndex:i];
 
         if ( [g isKindOfClass:[VGroup class]] ) // || [g isKindOfClass:[VPath class]] 
             cnt += [g countRecursive];
+        else
+            cnt++;
     }
     return cnt;
 }
@@ -173,11 +185,15 @@
         {   int	gCnt = [g countRecursive];
 
             if (ix >= cCnt && ix < cCnt+gCnt) // obj is inside g
-                return [g recursiveRemoveObjectAtIndex:(ix-cCnt)];
+            {   [g recursiveRemoveObjectAtIndex:(ix-cCnt)];
+                return;
+            }
             cCnt += [g countRecursive];
         }
         else if (cCnt == ix)
-            return [list removeObjectAtIndex:i];
+        {   [list removeObjectAtIndex:i];
+            return;
+        }
         else
             cCnt++;
     }
@@ -200,11 +216,15 @@
         {   int	gCnt = [g countRecursive];
 
             if (ix >= cCnt && ix < cCnt+gCnt) // obj is inside g
-                return [g recursiveInsertObject:obj atIndex:(ix-cCnt)];
+            {   [g recursiveInsertObject:obj atIndex:(ix-cCnt)];
+                return;
+            }
             cCnt += [g countRecursive];
         }
         else if (cCnt == ix)
-            return [list insertObject:obj atIndex:i];
+        {   [list insertObject:obj atIndex:i];
+            return;
+        }
         else
             cCnt++;
     }
@@ -366,7 +386,7 @@
 {   int	i, cnt;
 
     for (i=0, cnt = [list count]; i<cnt; i++)
-        [[list objectAtIndex:i] setWidth:w];
+        [(VGraphic*)[list objectAtIndex:i] setWidth:w];
 
     width = w;
     [self setUniColoringNew];
@@ -500,18 +520,7 @@
     [super setSelected:flag];
 }
 
-/* returns the old list
- */
-- (void)setList:(NSArray*)aList
-{
-    [list release];
-    list = [aList retain];
-    [self setColorNew];
-    coordBounds = bounds = NSZeroRect;
-    dirty = YES;
-}
-
-- (void)addObject:g
+- (void)addObject:(VGraphic*)g
 {
     [list addObject:g];
     [g setSelected:NO];
@@ -519,7 +528,7 @@
 
     /* check and set the colors */
     if ( ([list count] > 1 && uniColoring == NO) ||
-         ([g isKindOfClass:[VGroup class]] && [g uniColored] == NO) )
+         ([g isKindOfClass:[VGroup class]] && [(VGroup*)g uniColored] == NO) )
     {
         if (uniColoring == YES )
         {   /* init color */
@@ -554,7 +563,7 @@
         }
     }
     else // uniColoring == YES compare g colors with our
-    {   int		fild0 = 0, fild1 = 0;
+    {   int         fild0 = 0, fild1 = 0;
         NSColor		*col0, *fcol0 = nil, *ecol0 = nil, *col1, *fcol1 = nil, *ecol1 = nil;
         float		w0 = 0, ga0 = 0, w1 = 0, ga1 = 0;
         NSPoint		rc0 = {0.5,0.5}, rc1 = {0.5,0.5};
@@ -603,19 +612,17 @@
  * add a list to our list
  * after the operation we are selected, all of our objects are deselected
  */
-- (void)add:addList
+- (void)add:(NSArray*)addList
 {   int	i, cnt;
 
     if (!list)
         list = [[NSMutableArray allocWithZone:[self zone]] init];
-
     for (i=0, cnt=[addList count]; i<cnt; i++)
     {	id	g = [addList objectAtIndex:i];
 
         [g setSelected:NO];
         [list addObject:g];
     }
-
     [self setSelected:YES];
     coordBounds = bounds = NSZeroRect;
 }
@@ -714,7 +721,8 @@
         coordBounds = [[list objectAtIndex:cnt] coordBounds];
         for (i=cnt-1; i>=0; i--)
         {   rect = [[list objectAtIndex:i] coordBounds];
-            coordBounds = VHFUnionRect(rect, coordBounds);
+            if ( rect.size.width > 0.0 || rect.size.height > 0.0 )    // in case we have a bogus object
+                coordBounds = VHFUnionRect(rect, coordBounds);
         }
     }
     return coordBounds;
@@ -787,8 +795,8 @@
                 //[obj drawKnobs:rect direct:direct scaleFactor:scaleFactor];
                 [obj drawControls:rect direct:direct scaleFactor:scaleFactor];
                 [obj setSelected:sel];
-/*                if ([obj isMemberOfClass:[VCurve class]])
-                {   [NSBezierPath setDefaultLineWidth:0.0];
+                /*if ([obj isMemberOfClass:[VCurve class]])
+                {   [NSBezierPath setDefaultLineWidth:1.0/scaleFactor];
                     [NSBezierPath strokeLineFromPoint:[obj pointWithNum:0] toPoint:[obj pointWithNum:1]];
                     [NSBezierPath strokeLineFromPoint:[obj pointWithNum:3] toPoint:[obj pointWithNum:2]];
                 }*/
@@ -1057,6 +1065,15 @@
     dirty = YES;
 }
 
+- (void)transform:(NSAffineTransform*)matrix
+{   int	i;
+
+    for ( i=[list count]-1; i >= 0; i-- )
+        [[list objectAtIndex:i] transform:matrix];
+    coordBounds = bounds = NSZeroRect;
+    dirty = YES;
+}
+
 - (void)scale:(float)x :(float)y withCenter:(NSPoint)cp
 {   int		i;
 
@@ -1080,31 +1097,43 @@
     /* display intersections within path */
     for (i=0; i<(int)[list count]-1; i++)
     {	id	g = [list objectAtIndex:i];
-        NSPoint	*pts;
+        NSPoint	*pts = NULL;
         int	cnt, j;
 
         cnt = [g getIntersections:&pts with:[list objectAtIndex:i+1]];
         for (j=0; j<cnt; j++)
             CROSS_45(pts[j]);
-        free(pts);
+        if (pts)
+            free(pts);
     }
 #endif
 }
 
 /*
- * Check for a control point hit. No need to perform the hit detection in
- * the server since its a simple rectangle intersection check. Return the
- * point number hit in the pt_num argument.
+ * Check for a control point hit. Return the point number hit in the pt argument.
  * Does not set the graphic selection!!
  */
 - (BOOL)hitEdge:(NSPoint)p fuzz:(float)fuzz :(NSPoint*)pt :(float)controlsize
-{   int	i;
+{   int     i;
+    BOOL    gotHit = NO;
+    NSPoint hitP = p;
+    double  sqrDistBest = MAXFLOAT;
 
     for (i=[list count]-1; i>=0; i--)
-    {	id obj = [list objectAtIndex:i];
+    {	id  obj = [list objectAtIndex:i];
 
         if ([obj hitEdge:p fuzz:fuzz :pt :controlsize])
-            return YES;
+        {   gotHit = YES;
+            if ( SqrDistPoints(*pt, p) < sqrDistBest )
+            {   hitP = *pt;
+                sqrDistBest = SqrDistPoints(*pt, p);
+            }
+            //return YES;
+        }
+    }
+    if (gotHit)
+    {   *pt = hitP;
+        return YES;
     }
     return NO;
 }
@@ -1217,7 +1246,7 @@
 
 - (int)getIntersections:(NSPoint**)ppArray with:g
 {   int		i, j, iCnt = 0, ptsCnt = Min(100, [self numPoints]);
-    NSPoint	*pts;
+    NSPoint	*pts = NULL;
 
     *ppArray = malloc(ptsCnt * sizeof(NSPoint));
     for (i=[list count]-1; i>=0; i--)
@@ -1279,12 +1308,13 @@
                     (*ppArray)[iCnt++] = pts[j];
             }
         }
-        free(pts);
+        if (pts)
+            free(pts);
     }
 
     if (!iCnt)
     {	free(*ppArray);
-        *ppArray = 0;
+        *ppArray = NULL;
     }
 
     return iCnt;
@@ -1374,10 +1404,10 @@
 }*/
 #if 0
 - getIntersectionsAndSplittedObjects:(NSPoint**)ppArray :(int*)iCnt with:g
-{   int			i, j, lCnt = [list count], ptsCnt = Min(100, [self numPoints]);
-    NSPoint		*pts;
-    NSRect		gBounds = [g bounds];
-    NSMutableArray	*splitList = [NSMutableArray array], *spList = nil;
+{   int             i, j, lCnt = [list count], ptsCnt = Min(100, [self numPoints]);
+    NSPoint         *pts = NULL;
+    NSRect          gBounds = [g bounds];
+    NSMutableArray  *splitList = [NSMutableArray array], *spList = nil;
 
     *iCnt = 0;
     *ppArray = malloc(ptsCnt * sizeof(NSPoint));
@@ -1448,12 +1478,13 @@
                     (*ppArray)[(*iCnt)++] = pts[j];
             }
         }
-        free(pts);
+        if (pts)
+            free(pts);
     }
 
     if (!(*iCnt))
     {	free(*ppArray);
-        *ppArray = 0;
+        *ppArray = NULL;
     }
     if ( [splitList count] > [list count] )
         return splitList;
@@ -1461,11 +1492,11 @@
 }
 
 - uniteWith:(VGraphic*)ug
-{   int			i, j, iCnt, nothingRemoved = 0, cnt;
-    VPath		*ng;
-    NSMutableArray	*splitListG, *splitListUg;
-    NSPoint		*iPts;
-    NSAutoreleasePool	*pool;
+{   int                 i, j, iCnt, nothingRemoved = 0, cnt;
+    VPath               *ng;
+    NSMutableArray      *splitListG, *splitListUg;
+    NSPoint             *iPts = NULL;
+    NSAutoreleasePool   *pool;
 
     if ( ![ug isKindOfClass:[VPath class]] && ![ug isKindOfClass:[VArc class]]
         && ![ug isKindOfClass:[VRectangle class]] && ![ug isKindOfClass:[VGroup class]] )
@@ -1506,7 +1537,8 @@
         else
             [splitListUg addObject:[[ug copy] autorelease]];
     }
-    free(iPts);
+    if (iPts)
+        free(iPts);
 
     /* ug is'nt a path and not splitted now -> nothing to unite - NO */
     if ( ![ug isKindOfClass:[VPath class]] && [splitListUg count] == 1 )
@@ -1575,7 +1607,8 @@
     [aCoder encodeObject:fillColor];
     [aCoder encodeObject:endColor];
     [aCoder encodeValuesOfObjCTypes:"ff", &graduateAngle, &stepWidth];
-    [aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+    //[aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+    [aCoder encodePoint:radialCenter];          // 2012-01-08
 }
 - (id)initWithCoder:(NSCoder *)aDecoder
 {   int	version;
@@ -1592,7 +1625,8 @@
         fillColor = [[aDecoder decodeObject] retain];
         endColor = [[aDecoder decodeObject] retain];
         [aDecoder decodeValuesOfObjCTypes:"ff", &graduateAngle , &stepWidth];
-        [aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+        //[aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+        radialCenter = [aDecoder decodePoint];  // 2012-01-08
     }
     else
         [self setColorNew];
