@@ -1,7 +1,7 @@
 /* InspectorPanel.m
  * Cenon Inspector panel
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2011 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1993-10-01
@@ -43,9 +43,16 @@
 
 - (BOOL)canBecomeKeyWindow	{ return YES; }
 
+/* FIXME: this -init is not what it seems and should be renamed, it's only privatly used
+ */
 - init
 {   NSNotificationCenter	*notificationCenter = [NSNotificationCenter defaultCenter];
 
+    if ( ! defaultView )
+        defaultView = [[levView contentView] retain];
+    if ( !viewDict )
+        viewDict = [[NSMutableDictionary dictionary] retain];
+
     [self loadGraphic:nil];
     [self setDelegate:self];
 
@@ -90,8 +97,8 @@
 }
 
 - (void)loadGraphic:(id)g
-{   id		newWindow;
-    NSString	*string = @"ipCrosshairs.tiff";
+{   id          newWindow;
+    NSString    *string = @"ipCrosshairs.tiff";
 
     if (!defaultName)
         defaultName = [[self title] retain];
@@ -103,6 +110,7 @@
             if ( ![NSBundle loadModelNamed:@"IPLine3D" owner:self] )
                 NSLog(@"Cannot load IPLine3D model");
             [[line3DWindow init] setWindow:self];
+            [viewDict setObject:[line3DWindow contentView] forKey:[line3DWindow className]];
         }
         newWindow = line3DWindow;
         string = @"ipLine3D.tiff";
@@ -114,6 +122,7 @@
             if ( ![NSBundle loadModelNamed:@"IPLine" owner:self] )
                 NSLog(@"Cannot load IPLine model");
             [[lineWindow init] setWindow:self];
+            [viewDict setObject:[lineWindow contentView] forKey:[lineWindow className]];
         }
         newWindow = lineWindow;
         string = @"ipLine.tiff";
@@ -124,6 +133,7 @@
         {   if ( ![NSBundle loadModelNamed:@"IPPolyLine" owner:self] )
                 NSLog(@"Cannot load IPPolyLine model");
             [[polyLineWindow init] setWindow:self];
+            [viewDict setObject:[polyLineWindow contentView] forKey:[polyLineWindow className]];
         }
         newWindow = polyLineWindow;
         string = @"ipPolyLine.tiff";
@@ -134,6 +144,7 @@
         {   if ( ![NSBundle loadModelNamed:@"IPCurve" owner:self] )
                 NSLog(@"Cannot load IPCurve model");
             [[curveWindow init] setWindow:self];
+            [viewDict setObject:[curveWindow contentView] forKey:[curveWindow className]];
         }
         newWindow = curveWindow;
         string = @"ipCurve.tiff";
@@ -144,6 +155,7 @@
         {   if ( ![NSBundle loadModelNamed:@"IPArc" owner:self] )
                 NSLog(@"Cannot load IPArc model");
             [[arcWindow init] setWindow:self];
+            [viewDict setObject:[arcWindow contentView] forKey:[arcWindow className]];
         }
         newWindow = arcWindow;
         string = @"ipArc.tiff";
@@ -154,6 +166,7 @@
         {   if ( ![NSBundle loadModelNamed:@"IPThread" owner:self] )
                 NSLog(@"Cannot load IPThread model");
             [[threadWindow init] setWindow:self];
+            [viewDict setObject:[threadWindow contentView] forKey:[threadWindow className]];
         }
         newWindow = threadWindow;
         string = @"ipThread.tiff";
@@ -164,6 +177,7 @@
         {   if ( ![NSBundle loadModelNamed:@"IPPath" owner:self] )
                 NSLog(@"Cannot load IPPath model");
             [[pathWindow init] setWindow:self];
+            [viewDict setObject:[pathWindow contentView] forKey:[pathWindow className]];
         }
         newWindow = pathWindow;
         string = @"ipPath.tiff";
@@ -174,6 +188,7 @@
         {   if ( ![NSBundle loadModelNamed:@"IPText" owner:self] )
                 NSLog(@"Cannot load IPText model");
             [[textWindow init] setWindow:self];
+            [viewDict setObject:[textWindow contentView] forKey:[textWindow className]];
         }
         newWindow = textWindow;
         string = @"ipText.tiff";
@@ -187,12 +202,14 @@
             if ( ![NSBundle loadModelNamed:@"IPTextPath" owner:self] )
                 NSLog(@"Cannot load IPTextPath model");
             [[textPathWindow init] setWindow:self];
+            [viewDict setObject:[textPathWindow contentView] forKey:[textPathWindow className]];
         }
         if ([tpGraphic isKindOfClass:[VLine class]])
         {   if (!lineWindow)
             {   if ( ![NSBundle loadModelNamed:@"IPLine" owner:self] )
                     NSLog(@"Cannot load IPLine model");
                 [[lineWindow init] setWindow:self];
+                [viewDict setObject:[lineWindow contentView] forKey:[lineWindow className]];
             }
             tpWindow = lineWindow;
         }
@@ -202,6 +219,7 @@
             {   if ( ![NSBundle loadModelNamed:@"IPCurve" owner:self] )
                     NSLog(@"Cannot load IPCurve model");
                 [[curveWindow init] setWindow:self];
+                [viewDict setObject:[curveWindow contentView] forKey:[curveWindow className]];
             }
             tpWindow = curveWindow;
         }
@@ -211,9 +229,12 @@
             {   if ( ![NSBundle loadModelNamed:@"IPArc" owner:self] )
                     NSLog(@"Cannot load IPArc model");
                 [[arcWindow init] setWindow:self];
+                [viewDict setObject:[arcWindow contentView] forKey:[arcWindow className]];
             }
             tpWindow = arcWindow;
         }
+        if ( [viewDict objectForKey:[tpWindow className]] != [tpWindow contentView] )
+            [tpWindow setContentView:[viewDict objectForKey:[tpWindow className]]];  // GNUstep removes the contentview
         tpView = [tpWindow contentView];
         [[textPathWindow pathView] setContentView:[tpView retain]];
         [tpView setAutoresizingMask:0]; // NSViewHeightSizable
@@ -228,6 +249,7 @@
         {   if ( ![NSBundle loadModelNamed:@"IPGroup" owner:self] )
                 NSLog(@"Cannot load IPGroup model");
             [[groupWindow init] setWindow:self];
+            [viewDict setObject:[groupWindow contentView] forKey:[groupWindow className]];
         }
         newWindow = groupWindow;
         string = @"ipGroup.tiff";
@@ -238,6 +260,7 @@
         {   if ( ![NSBundle loadModelNamed:@"IPRectangle" owner:self] )
                 NSLog(@"Cannot load IPRectangle model");
             [[rectangleWindow init] setWindow:self];
+            [viewDict setObject:[rectangleWindow contentView] forKey:[rectangleWindow className]];
         }
         newWindow = rectangleWindow;
         string = @"ipRectangle.tiff";
@@ -248,6 +271,7 @@
         {   if ( ![NSBundle loadModelNamed:@"IPImage" owner:self] )
                 NSLog(@"Cannot load IPImage model");
             [[imageWindow init] setWindow:self];
+            [viewDict setObject:[imageWindow contentView] forKey:[imageWindow className]];
         }
         newWindow = imageWindow;
         string = @"ipImage.tiff";
@@ -258,6 +282,7 @@
         {   if ( ![NSBundle loadModelNamed:@"IPMark" owner:self] )
                 NSLog(@"Cannot load IPMark model");
             [[markWindow init] setWindow:self];
+            [viewDict setObject:[markWindow contentView] forKey:[markWindow className]];
         }
         newWindow = markWindow;
         string = @"ipMark.tiff";
@@ -268,6 +293,7 @@
         {   if ( ![NSBundle loadModelNamed:@"IPWeb" owner:self] )
                 NSLog(@"Cannot load IPWeb model");
             [[webWindow init] setWindow:self];
+            [viewDict setObject:[webWindow contentView] forKey:[webWindow className]];
         }
         newWindow = webWindow;
         string = @"ipWeb.tiff";
@@ -278,6 +304,7 @@
         {   if ( ![NSBundle loadModelNamed:@"IPSinking" owner:self] )
                 NSLog(@"Cannot load IPSinking model");
             [[sinkingWindow init] setWindow:self];
+            [viewDict setObject:[sinkingWindow contentView] forKey:[sinkingWindow className]];
         }
         newWindow = sinkingWindow;
         string = @"ipSinking.tiff";
@@ -288,6 +315,7 @@
         {   if ( ![NSBundle loadModelNamed:@"IPCrosshairs" owner:self] )
                 NSLog(@"Cannot load IPCrosshairs model");
             [[crosshairsWindow init] setWindow:self];
+            [viewDict setObject:[crosshairsWindow contentView] forKey:[crosshairsWindow className]];
         }
         newWindow = crosshairsWindow;
         g = [(DocView*)[self view] origin];
@@ -307,19 +335,22 @@
     graphic = g;
 
     if (objectWindow != newWindow)
-    {	NSImage	*image = [NSImage imageNamed:string]; 
+    {	NSImage	*image = [NSImage imageNamed:string];
 
         [[levRadio cellAtRow:0 column:0] setImage:image];
     }
     objectWindow = newWindow;
+
     [self setTitle:[objectWindow title]];
 
-    if (![levRadio selectedColumn])
+    if ( [levRadio selectedColumn] == 0 && newWindow != activeWindow )
     {
-        if (newWindow != activeWindow && [activeWindow respondsToSelector:@selector(displayWillEnd)])
+        if ([activeWindow respondsToSelector:@selector(displayWillEnd)])
             [activeWindow displayWillEnd];
+        if ( [viewDict objectForKey:[objectWindow className]] != [objectWindow contentView] )
+            [objectWindow setContentView:[viewDict objectForKey:[objectWindow className]]];  // GNUstep removes the contentview
+        [self setLevelView:[objectWindow contentView]];
         activeWindow = objectWindow;
-        [self setLevelView:[activeWindow contentView]];
     }
     [activeWindow update:g];
     if ( [self isVisible] )
@@ -384,6 +415,8 @@
     switch (level)
     {
         case IP_OBJECT:
+            if ( [viewDict objectForKey:[objectWindow className]] != [objectWindow contentView] )
+                [objectWindow setContentView:[viewDict objectForKey:[objectWindow className]]];   // GNUstep workaround
             return objectWindow;
         case IP_STROKEWIDTH:
             if (!allStrokeWindow)
@@ -391,7 +424,10 @@
                 if (![NSBundle loadModelNamed:@"IPAllStrokeWidth" owner:self])
                     NSLog(@"Cannot load IPAllStrokeWidth model");
                 [[allStrokeWindow init] setWindow:self];
+                [viewDict setObject:[allStrokeWindow contentView] forKey:[allStrokeWindow className]];
             }
+            else if ( [viewDict objectForKey:[allStrokeWindow className]] != [allStrokeWindow contentView] )
+                [allStrokeWindow setContentView:[viewDict objectForKey:[allStrokeWindow className]]];   // GNUstep workaround
             return allStrokeWindow;
         case IP_FILLING:
             if (!allFillingWindow)
@@ -399,7 +435,10 @@
                 if (![NSBundle loadModelNamed:@"IPAllFilling" owner:self])
                     NSLog(@"Cannot load IPAllFilling model");
                 [[allFillingWindow init] setWindow:self];
+                [viewDict setObject:[allFillingWindow contentView] forKey:[allFillingWindow className]];
             }
+            else if ( [viewDict objectForKey:[allFillingWindow className]] != [allFillingWindow contentView] )
+                [allFillingWindow setContentView:[viewDict objectForKey:[allFillingWindow className]]];   // GNUstep workaround
             return allFillingWindow;
         /*case IP_TEXT:
             if (!allTextWindow)
@@ -415,7 +454,10 @@
                 if (![NSBundle loadModelNamed:@"IPAllAcc" owner:self])
                     NSLog(@"Cannot load IPAllAcc model");
                 [[allAccWindow init] setWindow:self];
+                [viewDict setObject:[allAccWindow contentView] forKey:[allAccWindow className]];
             }
+            else if ( [viewDict objectForKey:[allAccWindow className]] != [allAccWindow contentView] )
+                [allAccWindow setContentView:[viewDict objectForKey:[allAccWindow className]]];   // GNUstep workaround
             return allAccWindow;
         case IP_LAYERS:
             if (!allLayersWindow)
@@ -423,7 +465,10 @@
                 if (![NSBundle loadModelNamed:@"IPAllLayers" owner:self])
                     NSLog(@"Cannot load IPAllLayers model");
                 [[allLayersWindow init] setWindow:self];
+                [viewDict setObject:[allLayersWindow contentView] forKey:[allLayersWindow className]];
             }
+            else if ( [viewDict objectForKey:[allLayersWindow className]] != [allLayersWindow contentView] )
+                [allLayersWindow setContentView:[viewDict objectForKey:[allLayersWindow className]]];   // GNUstep workaround
             return allLayersWindow;
         default: return nil;
     }
@@ -431,7 +476,10 @@
 
 - (void)setLevelView:theView
 {
-    [levView setContentView:[theView retain]];
+    if ( ! defaultView )
+        defaultView = [[levView contentView] retain];
+    if ( theView != [levView contentView] ) // GNUstep can't handle setting the same contentView twice
+        [levView setContentView:(theView) ? [theView retain] : defaultView];
     [theView setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable];
 
     [self display];
