@@ -1,11 +1,12 @@
 /* UpdateController.m
  * Checking for Updates...
  *
- * Copyright 2010-2011 by vhf interservice GmbH
+ * Copyright 2010-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  2010-05-27
- * modified: 2011-03-30 (-checkForUpdates: test for Prefs_DisableAutoUpdate)
+ * modified: 2012-02-07 (-connectionDidFinishLoading: use -writeToFile:...encoding:error:)
+ *           2011-03-30 (-checkForUpdates: test for Prefs_DisableAutoUpdate)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -33,6 +34,7 @@
 #include "messages.h"
 #include "CenonModuleMethods.h"
 #include "PreferencesMacros.h"  // Prefs_DisableAutoUpdate
+#include <VHFShared/VHFStringAdditions.h>       // -writeToFile:...
 
 static UpdateController *sharedInstance = nil;  // default object
 static NSString         *checkMarkStr = nil;    // the checkmark
@@ -170,7 +172,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
                                   (serial) ? serial : @"", (netId) ? netId : @"", nil]
                           forKey:name];
         if ( !isAutoCheck )
-            printf("installedModules: %s = %s (%s) %s %s\n", [name UTF8String], [version UTF8String],
+            printf("Installed Modules: %s = %s (%s) %s %s\n", [name UTF8String], [version UTF8String],
                (date) ? [date UTF8String] : "", (serial) ? [serial UTF8String] : "",
                (netId) ? [netId UTF8String] : "");
     }
@@ -195,7 +197,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     if ( isAutoCheck && Prefs_DisableAutoUpdate )
         return; // automatic check is disabled in preferences
     if ( isAutoCheck && lastUpdateCheck &&
-        [[NSDate date] timeIntervalSinceDate:lastUpdateCheck] < 3600*24*7 )
+         [[NSDate date] timeIntervalSinceDate:lastUpdateCheck] < 3600*24*7 )
         return; // not yet one week -> no check
 
     installedModules = [self installedModules];
@@ -293,6 +295,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     [checkOrder addObject:appName];
     if ( !isAutoCheck )
         printf("Installed App:     %s = %s (%s)\n", [appName UTF8String], [appVersion UTF8String], [appDate UTF8String]);
+    // FIXME: this is basically duplicating -installedModules and should be united
     for ( i=0; i<[modules count]; i++ ) // modules
     {   NSBundle        *bundle = [modules objectAtIndex:i]; // our loaded modules
         NSDictionary    *infoDict = [bundle infoDictionary];
@@ -309,7 +312,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     }
     if ( [updateDict objectForKey:@"order"] )
         checkOrder = [updateDict objectForKey:@"order"];
-    /* Check for install paths of items given in update.plist and add their versions etc. */
+    /* Check for install paths of additional items given in update.plist and add their versions etc. */
     for ( i=0; i<[updateKeys count]; i++ )
     {   NSString        *key = [updateKeys objectAtIndex:i], *path, *pathU = nil;
         NSDictionary    *uDict = [updateDict objectForKey:key];
@@ -386,6 +389,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
                 v = @"0.0.0"; // this is old anyway
             if ( [[iArray objectAtIndex:0] compare:(id)v options:NSNumericSearch] == NSOrderedAscending )   // installed < version
                 updateAvailable = YES;
+            // TODO: if version is the same, compare date
             else    // installed >= version
                 continue;
         }
@@ -403,7 +407,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
                 if ( ![uDictI isKindOfClass:[NSDictionary class]] )
                     continue;
                 /* if something is installed, then we installed a Cenon-Package already */
-                if ( [[dataDict objectForKey:@"updateKey"] count] )
+                if ( [(NSArray*)[dataDict objectForKey:@"updateKey"] count] )
                 {   installModule = YES;
                     url = [uDict objectForKey:@"md"];
                     version = mVersion;
@@ -429,8 +433,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
 
             price = nil;
             if ( priceDict )
-            {   //NSString    *v = ([uDict objectForKey:@"mv"]) ? [uDict objectForKey:@"mv"] : version;
-                NSString    *v = ([iArray count]) ? [iArray objectAtIndex:0] : appVersion;  // version of installed module
+            {   NSString    *v = ([uDict objectForKey:@"mv"]) ? [uDict objectForKey:@"mv"] : version;
                 NSRange     range = [v rangeOfString:@"." options:NSBackwardsSearch];
 
                 if ( range.length )
@@ -457,7 +460,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
 
     /* Add additional items for installed items (Fonts, Ephemeris, ...)
      */
-    for ( i=0; i<[[dataDict objectForKey:@"updateKey"] count]; i++ )
+    for ( i=0; i<[(NSArray*)[dataDict objectForKey:@"updateKey"] count]; i++ )
     {   NSString        *key = [[dataDict objectForKey:@"updateKey"] objectAtIndex:i];
         NSDictionary    *uDict  = [updateDict objectForKey:key];    // v, s, l, i, n, r
         NSArray         *relArray = [uDict objectForKey:@"r"];
@@ -582,10 +585,11 @@ static NSString         *checkMarkStr = nil;    // the checkmark
                 if ( ![array count] && NSAppKitVersionNumber >= NSAppKitVersionNumber10_4)
                     array = NSSearchPathForDirectoriesInDomains(NSDesktopDirectory, NSUserDomainMask, YES);
 #endif
-                if ( array && [array count] )   // $HOME/Downloads/PACKAGE
+                if ( array && [array count] )   // 1. $HOME/Downloads/PACKAGE
                     path = [[array objectAtIndex:0] stringByAppendingPathComponent:fileName];
-                if ( !path )                    // /tmp/PACKAGE
-                    //path = vhfPathWithPathComponents(NSTemporaryDirectory(), fileName, nil);   // this is a nonsense folder: "/var/folders/8L/arg/-Tmp-"
+                if ( !path )                    // 2. /var/folders/8L/.../-Tmp-/PACKAGE
+                    path = vhfPathWithPathComponents(NSTemporaryDirectory(), fileName, nil);   // this is a nonsense folder: "/var/folders/8L/arg/-Tmp-"
+                if ( !path )                    // 3. /tmp/PACKAGE
                     path = vhfPathWithPathComponents(NSOpenStepRootDirectory(), @"tmp", fileName, nil);
                 fileCnt ++;
                 [downloadFiles addObject:[NSArray arrayWithObjects:url, path, nil]];    // url, path
@@ -676,7 +680,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
         NSString        *url = [[updateDict objectForKey:[keyArray objectAtIndex:rowIx]] objectForKey:@"d"];
 
         if ( url && // checkmark only for downloadable items
-             (![[colArray objectAtIndex:rowIx] length] || ![[colArray objectAtIndex:rowIx] intValue]) )
+             (![(NSString*)[colArray objectAtIndex:rowIx] length] || ![[colArray objectAtIndex:rowIx] intValue]) )
             boolStr = @"1";
         [colArray replaceObjectAtIndex:rowIx withObject:boolStr];
     }
@@ -710,7 +714,9 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     {   NSString    *updateFile;
 
         updateFile = vhfPathWithPathComponents(vhfUserLibrary(APPNAME), @".update.plist", nil);
-        [string writeToFile:updateFile atomically:NO];
+        //[string writeToFile:updateFile atomically:NO];
+        [string writeToFile:updateFile atomically:NO
+                   encoding:NSUTF8StringEncoding error:NULL];   // >= 10.5
         [self checkForUpdateAndDisplayPanel];
     }
     // TODO: show "up to date" info
@@ -741,7 +747,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     sizeTotal = [response expectedContentLength];
     sizeDownl = 0;
 }
-- (void)download:(NSURLDownload *)download didReceiveDataOfLength:(unsigned)length
+- (void)download:(NSURLDownload *)download didReceiveDataOfLength:(NSUInteger)length
 {  
     sizeDownl += length;
     if (sizeTotal > 0)
@@ -815,7 +821,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
 
             for ( i=0, path=nil; i<[downloadFiles count]; i++)
             {   path = [[downloadFiles objectAtIndex:i] objectAtIndex:1];
-                if ( [[path pathExtension] isEqual:@"tar"] && [fileManager fileExistsAtPath:path] ) // del TAR-file
+                if ( [fileManager fileExistsAtPath:path] )  // delete TAR-file
                     [fileManager removeFileAtPath:path handler:nil];
                 // TODO: the following must not happen before the files are really installed !
                 /*path = [path stringByDeletingPathExtension];
@@ -981,7 +987,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
 {
     [dataDict release];
     dataDict = [newDataDict retain];
-    rowCount = [[dataDict objectForKey:@"name"] count];
+    rowCount = [(NSArray*)[dataDict objectForKey:@"name"] count];
 }
 - (NSDictionary*)dataDict
 {
