@@ -1,11 +1,20 @@
 /* DXFImport.m
  * DXF import object
  *
- * Copyright (C) 1996-2007 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-05-01
- * modified: 2007-01-30 (close PolyLine: check for start == end point)
+ * modified: 2012-02-04 (-scanGroup: transform coordinates from OCS to WCS, if necessary)
+ *           2012-02-03 (InternalToDeviceRes(), DeviceResToInternal(): float -> double)
+ *           2011-12-11 (-getGraphicFromData: TODO for SPLINEs added)
+ *           2011-04-04 (-getGraphicFromData: 3DFACE TODO-log added)
+ *           2011-02-27 (-init: initialize group.handle, state.color, blockScanner, rable, vivibleList)
+ *           2010-10-31 (buildArc(): dx, dy calced with double)
+ *           2010-09-11 (-importDXF: try UTF8 and Latin encodings to make GNUstep happy)
+ *           2010-06-11 (buildArc(): 360.0 - acos(cn), was 180.0 - )
+ *           2010-04-03 (-getArc: negative 360 deg arcs work now, buildArc(): dx == 0 case)
+ *           2007-01-30 (close PolyLine: check for start == end point)
  *
  * This file is part of the vhf Import Library.
  *
@@ -46,8 +55,8 @@
 //#define	SqrDistPoints(p1, p2)	(((p1).x-(p2).x)*((p1).x-(p2).x)+((p1).y-(p2).y)*((p1).y-(p2).y))
 
 /* r in points/inch */
-#define	InternalToDeviceRes(a, r)	((float)(a) * (float)(r) / 72.0)
-#define	DeviceResToInternal(a, r)	((float)(a) * 72.0 / (float)(r))
+#define	InternalToDeviceRes(a, r)	((double)(a) * (double)(r) / 72.0)
+#define	DeviceResToInternal(a, r)	((double)(a) * 72.0 / (double)(r))
 
 static NSCharacterSet	*newLineSet, *skipSet;
 
@@ -58,7 +67,7 @@ static BOOL gotoTable(NSScanner *scanner, NSString *name);
 static BOOL gotoGroup( NSScanner *scanner, NSString *name);
 static NSArray *getTableFromData(NSString *data);
 static NSColor *getColorFromTable(NSString *str, NSArray *table);
-static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *angle);
+static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *angle);
 
 @interface DXFImport(PrivateMethods)
 - (BOOL)getHeader:(NSString*)data;
@@ -67,6 +76,7 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
 - (BOOL)getGraphicFromData:(NSScanner*)scanner :cList;
 - (BOOL)scanGroup:(NSScanner*)scanner;
 - (BOOL)getLine:(NSScanner*)scanner :(NSPoint*)p0 :(NSPoint*)p1 :(NSColor**)color;
+- (BOOL)getLine3D:(NSScanner*)scanner :(V3Point*)p0 :(V3Point*)p1 :(NSColor**)color;
 - (BOOL)getLWPolyline:(NSScanner*)scanner addToList:(NSMutableArray*)list;
 - (BOOL)getPolyline:(NSScanner*)scanner;
 - (BOOL)getVertex:(NSScanner*)scanner :(int*)kind :(NSPoint*)p0 :(NSPoint*)p1 :(float*)angle;
@@ -74,6 +84,7 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
 - (BOOL)getCircle:(NSScanner*)scanner :(NSPoint*)ctr :(NSPoint*)start :(float*)angle;
 - (BOOL)getArc:(NSScanner*)scanner :(NSPoint*)ctr :(NSPoint*)start :(float*)angle;
 - (BOOL)getText :(NSScanner*)scanner mtext:(BOOL)mext :(NSString**)string :(float*)angle :(NSPoint*)origin :(float*)size :(float*)ar :(int*)alignment;
+- (BOOL)get3DFace:(NSScanner*)scanner points:(V3Point*)pts color:(NSColor**)color;
 - (BOOL)getInsert:(NSScanner*)scanner :cList;
 - (void)updateBounds:(NSPoint)p;
 
@@ -212,14 +223,14 @@ static BOOL gotoTable(NSScanner *scanner, NSString *name)
  * modified:  2001-02-23
  *
  * purpose:   get tables from data
- * 	      at the moment we only read the layer table...
+ *            at the moment we only read the layer table...
  * parameter: data (dxf file)
  */
 static NSArray *getTableFromData(NSString *data)
-{   NSScanner		*scanner = [NSScanner scannerWithString:data];
-    NSString		*string;
-    NSMutableArray	*table = [NSMutableArray array];
-    NSMutableDictionary	*dict = nil;
+{   NSScanner           *scanner = [NSScanner scannerWithString:data];
+    NSString            *string;
+    NSMutableArray      *table = [NSMutableArray array];
+    NSMutableDictionary *dict = nil;
 
     [scanner setCharactersToBeSkipped:skipSet];
 
@@ -412,6 +423,13 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
     group.text = nil;
     group.name = nil;
     group.layer = nil;
+    group.handle = nil;
+
+    state.color = nil;
+
+    blockScanner = nil;
+    table = nil;
+    visibleList = nil;
 
     return self;
 }
@@ -422,12 +440,17 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 }
 
 /* created:   1996-01-25
- * modified:  2002-10-26
+ * modified:  2010-09-11 (load UTF8 instead of ASCII, if it fails try Latin)
  * parameter: dxfData	the DXF data stream
  * purpose:   start interpretation of dxfData
  */
 - (id)importDXF:(NSData*)dxfData
-{   NSString	*data = [[[NSString alloc] initWithData:dxfData encoding:NSASCIIStringEncoding] autorelease];
+{   NSString	*data = [[[NSString alloc] initWithData:dxfData
+                                            encoding:NSUTF8StringEncoding] autorelease];
+
+    if ( ! data )   // try latin
+        data = [[[NSString alloc] initWithData:dxfData
+                                      encoding:NSISOLatin1StringEncoding] autorelease];
 
     newLineSet = [NSCharacterSet characterSetWithCharactersInString:@"\r\n"];
     skipSet = [NSCharacterSet characterSetWithCharactersInString:@" \t"];
@@ -512,9 +535,9 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 }
 
 - (BOOL)interpret:(NSString*)data
-{   NSMutableArray	*cList;		// current list
-    NSRect		bounds;
-    NSScanner		*scanner = [NSScanner scannerWithString:data];
+{   NSMutableArray  *cList;		// current list
+    NSRect          bounds;
+    NSScanner       *scanner = [NSScanner scannerWithString:data];
 
     /* init bounds */
     ll.x = ll.y = LARGE_COORD;
@@ -554,8 +577,8 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
  * modified: 2005-09-26 (close polyline with arc)
  */
 - (BOOL)getGraphicFromData:(NSScanner*)scanner :(id)cList
-{   NSPoint	p0, p1;
-    NSString	*name;
+{   NSPoint     p0, p1;
+    NSString    *name;
 
     /* goto name line */
     [scanner scanUpToCharactersFromSet:newLineSet intoString:NULL];
@@ -568,7 +591,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 
     switch ( state.mode )
     {
-	case MODE_NORMAL:
+        case MODE_NORMAL:
             /* line */
             if ( [name isEqual:GRPLINE] )
             {
@@ -620,10 +643,10 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
             }
             /* text, mtext */
             else if ( [name isEqual:GRPTEXT] || [name isEqual:GRPMTEXT] )
-            {	NSPoint		origin;
-                float		angle, size, ar;
-                int		alignment;
-                NSString	*string = nil;
+            {	NSPoint     origin;
+                float       angle, size, ar;
+                int         alignment;
+                NSString    *string = nil;
 
                 if ([self getText:scanner mtext:[name isEqual:GRPMTEXT]
                                  :&string :&angle :&origin :&size :&ar :&alignment])
@@ -642,12 +665,34 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
             {
                 [self getInsert:scanner :cList];
             }
+            /* SPLINE */
+            else if ( [name isEqual:GRPSPLINE] )
+            {
+                // TODO: SPLINEs (see example Stebner_SPLINEs*.dxf)
+                //if ([self getSpline:scanner :pts :&state.color])  // we should return/convert to bezier curve(s)
+                //    [self addCurve:pts toLayer:group.layer];
+                printf("TODO, DXF-Import: SPLINEs not implemented\n");
+            }
+            /* 3D LINE */
+            else if ( [name isEqual:GRP3DLINE] )
+            {   V3Point p0, p1;
+
+                if ([self getLine3D:scanner :&p0 :&p1 :&state.color])
+                    [self addLine3D:p0 :p1 toLayer:group.layer];
+            }
+            /* 3D FACE */
+            else if ( [name isEqual:GRP3DFACE] )
+            {   V3Point pts[4];
+
+                if ([self get3DFace:scanner points:pts color:&state.color] )
+                    [self add3DFace:pts toLayer:group.layer];
+            }
             break;
         case MODE_VERTEX:
             /* vertex */
             if ( [name isEqual:GRPVERTEX] )
-            {	int	kind;
-                float	a;
+            {	int     kind;
+                float   a;
 
                 [self getVertex:scanner :&kind :&p0 :&p1 :&a];
                 //printf("kind=%d p0={%.0f %.0f} p1={%.0f %.0f} a=%.0f\n", kind, p0.x, p0.y, p1.x, p1.y, a);
@@ -738,6 +783,37 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
     return YES;
 }
 
+/* created:   2011-04-04
+ * modified:  
+ *
+ * purpose:   get 3D line
+ * parameter: scanner (the dxf data)
+ *            g (the destination for the element)
+ *            state
+ *            parms
+ */
+- (BOOL)getLine3D:(NSScanner*)scanner :(V3Point*)p0 :(V3Point*)p1 :(NSColor**)color
+{
+    [self scanGroup:scanner];
+    if ( [visibleList count] && ![visibleList containsObject:group.handle])
+        return NO;
+
+    *color = [self currentColor];
+    p0->x = DeviceResToInternal(group.x0, res) + state.offset.x;
+    p0->y = DeviceResToInternal(group.y0, res) + state.offset.y;
+    p0->z = DeviceResToInternal(group.z0, res) /*+ state.offset.z*/;
+    p1->x = DeviceResToInternal(group.x1, res) + state.offset.x;
+    p1->y = DeviceResToInternal(group.y1, res) + state.offset.y;
+    p1->z = DeviceResToInternal(group.z1, res) /*+ state.offset.z*/;
+
+    state.width = 0.0;
+
+    [self updateBounds:NSMakePoint(p0->x, p0->y)];
+    [self updateBounds:NSMakePoint(p1->x, p1->y)];
+
+    return YES;
+}
+
 /* created:   2001-02-22
  * modified:  2006-08-10 (check for close with & operator)
  *
@@ -1009,30 +1085,46 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
     return YES;
 }
 
-static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *angle)
-{   NSPoint	p, m;
-    double	l, ang, dy, r, an, h, sn;
+/* Note: see page 323 of DXF book
+ * modified: 2010-10-31 (dx, dy calced with double)
+ *           2010-06-11 (360 - acos(cn))
+ *           2010-04-03
+ */
+static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *angle)
+{   NSPoint	p;
+    double	l, ang, dx, dy, r, an, h;
+    double  mx, my, begx = beg.x, begy = beg.y, endx = end.x, endy = end.y;
 
     ang = 4.0 * RadToDeg(atan( a ));
-    l = sqrt(SqrDistPoints(beg, end));
-    h = (a*l)/2.0;
-    m.x = (beg.x+end.x)/2.0; m.y = (beg.y+end.y)/2.0;
-    dy = m.y - beg.y;
-    //if (h>0)	/* dont know if this is correct */
+    l = sqrt(SqrDistPoints(beg, end));  // length of arc from point to point
+    h = (a*l)/2.0;                      // height of arc
+    mx = (begx+endx)/2.0; my = (begy+endy)/2.0; // mit point between start and end
+    dx = mx - begx;                     // 
+    dy = my - begy;                     // 
+    //if (h>0)	/* don't know if this is correct */
     //    ang = -ang;
 //printf("h:%f ang:%f\n", h, ang);
     /* asin(dy/(l/2)) = angle from 0 degree to l, we may use atan(2h/l) instead */
-    sn = Max( -1.0, Min( 1.0, dy / (l/2.0) ) );
-    if ( beg.x>end.x )
-        an = 180.0 - (RadToDeg(asin(sn)) - (((ang<0) ? -90.0 : 90.0)-ang/2.0));
-    else
-        an = RadToDeg(asin(sn)) + (((ang<0) ? -90.0 : 90.0)-ang/2.0);
+    if ( Abs(dx) < TOLERANCE )              // larger y component (vertical arc) (only tested for vertical case !)
+    {   double  cn = Max( -1.0, Min( 1.0, dx / (l/2.0) ) );
+        if ( beg.y > end.y )
+            an = 360.0 - (RadToDeg(acos(cn)) - (((ang<0) ? -90.0 : 90.0)-ang/2.0));
+        else
+            an = RadToDeg(acos(cn)) + (((ang<0) ? -90.0 : 90.0)-ang/2.0);
+    }
+    else                                    // larger x component
+    {   double  sn = Max( -1.0, Min( 1.0, dy / (l/2.0) ) );
+        if ( beg.x > end.x )
+            an = 180.0 - (RadToDeg(asin(sn)) - (((ang<0) ? -90.0 : 90.0)-ang/2.0));
+        else
+            an = RadToDeg(asin(sn)) + (((ang<0) ? -90.0 : 90.0)-ang/2.0);
+    }
     r = Abs(l/(2.0*sin(DegToRad(ang/2.0))));
 //NSLog(@"0 h=%f an=%f dy=%f l=%f ang=%f r=%f", h, an, dy, l, ang, r);
     p.x = beg.x + r; p.y = beg.y;
     vhfRotatePointAroundCenter(&p, beg, an);
     *ctr = p;
-    *angle = ang;
+    *angle = (float)ang;
 }
 
 /* created:   18.05.93
@@ -1089,9 +1181,9 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
  * modified:  2004-08-04
  *
  * purpose:   get circle
- * parameter: scanner	the dxf data
- *            ctr	center
- *            start	
+ * parameter: scanner   the dxf data
+ *            ctr       center
+ *            start
  *            angle	360 degree
  */
 - (BOOL)getCircle:(NSScanner*)scanner :(NSPoint*)ctr :(NSPoint*)start :(float*)angle
@@ -1119,7 +1211,7 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
 }
 
 /* created:   1993-05-17
- * modified:  2004-08-04
+ * modified:  2010-04-03 (negative 360 deg arcs work now)
  *
  * purpose:   get arc
  * parameter: cp (the dxf data)
@@ -1138,17 +1230,22 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
     state.color = [self currentColor];
     ctr->x = DeviceResToInternal(group.x0, res) + state.offset.x;
     ctr->y = DeviceResToInternal(group.y0, res) + state.offset.y;
+    //FIXME: group.extX, group.extY, group.extZ
     r = DeviceResToInternal(group.width, res);
     ba = group.begAngle;
     ea = group.endAngle;
 
     start->x = ctr->x + r;
     start->y = ctr->y;
-    if (ba>=360.0) ba-=360.0;
-    if (ea<ba) ea+=360.0;
+    *angle = 0.0;
+    if ( Abs(ea - ba) >= 360.0 )
+        *angle = ea - ba;
+    if (ba >= 360.0) ba -= 360.0;
+    if (ea < ba)     ea += 360.0;
     vhfRotatePointAroundCenter(start, *ctr, ba);
 
-    *angle = ea - ba;
+    if ( *angle == 0.0 )
+        *angle = ea - ba;
 
     state.width = 0.0;
 
@@ -1246,6 +1343,55 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
     return YES;
 }
 
+/* created:   2011-04-04
+ * modified:  2011-04-04
+ *
+ * purpose:   get insert
+ * parameter: scanner (the dxf data)
+ *            layer
+ * 3DFACE    Four points defining the corners of the face: (10, 20, 30),
+ * (11, 21, 31), (12, 22, 32), and (13, 23, 33).  70 (invisible
+ * edge flags -optional 0).  If only three points were entered
+ * (forming a triangular face), the third and fourth points will
+ * be the same.  The meanings of the bit-coded "invisible edge
+ * flags" are shown in the following table.
+ *
+ * Flag bit value           Meaning
+ * 1        First edge is invisible
+ * 2        Second edge is invisible
+ * 4        Third edge is invisible
+ * 8        Fourth edge is invisible
+ */
+- (BOOL)get3DFace:(NSScanner*)scanner points:(V3Point*)pts color:(NSColor**)color
+{
+    [self scanGroup:scanner];
+    if ( [visibleList count] && ![visibleList containsObject:group.handle])
+        return NO;
+
+    *color = [self currentColor];
+    pts[0].x = DeviceResToInternal(group.x0, res) + state.offset.x;
+    pts[0].y = DeviceResToInternal(group.y0, res) + state.offset.y;
+    pts[0].z = DeviceResToInternal(group.z0, res) /*+ state.offset.z*/;
+    pts[1].x = DeviceResToInternal(group.x1, res) + state.offset.x;
+    pts[1].y = DeviceResToInternal(group.y1, res) + state.offset.y;
+    pts[1].z = DeviceResToInternal(group.z1, res) /*+ state.offset.z*/;
+    pts[2].x = DeviceResToInternal(group.x2, res) + state.offset.x;
+    pts[2].y = DeviceResToInternal(group.y2, res) + state.offset.y;
+    pts[2].z = DeviceResToInternal(group.z2, res) /*+ state.offset.z*/;
+    pts[3].x = DeviceResToInternal(group.x3, res) + state.offset.x;
+    pts[3].y = DeviceResToInternal(group.y3, res) + state.offset.y;
+    pts[3].z = DeviceResToInternal(group.z3, res) /*+ state.offset.z*/;
+
+    state.width = 0.0;
+
+    [self updateBounds:NSMakePoint(pts[0].x, pts[0].y)];
+    [self updateBounds:NSMakePoint(pts[1].x, pts[1].y)];
+    [self updateBounds:NSMakePoint(pts[2].x, pts[2].y)];
+    [self updateBounds:NSMakePoint(pts[3].x, pts[3].y)];
+
+    return YES;
+}
+
 /* created:   30.09.94
  * modified:  30.09.94 10.03.97
  *
@@ -1253,13 +1399,13 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
  * parameter: scanner (the dxf data)
  *            cList
  */
-- (BOOL)getInsert:(NSScanner*)scanner :cList
-{   NSPoint		point;				/* position of the text */
-    NSString		*name, *string;
-    float		angle;
-    NSScanner		*myBlockScanner = nil;
-    int			location;
-    NSAutoreleasePool	*pool = [[NSAutoreleasePool alloc] init];
+- (BOOL)getInsert:(NSScanner*)scanner :(id)cList
+{   NSPoint             point;				/* position of the text */
+    NSString            *name, *string;
+    float               angle;
+    NSScanner           *myBlockScanner = nil;
+    int                 location;
+    NSAutoreleasePool   *pool = [[NSAutoreleasePool alloc] init];
 
     [self scanGroup:scanner];
     if ( [visibleList count] && ![visibleList containsObject:group.handle])
@@ -1366,13 +1512,63 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
     return color;
 }
 
+/* arbitrary axis algorithm - transform OCS (Object) to WCS (World)
+ * build the cross product
+ *
+ * W        our coordinate             [WCS]
+ * AZ (N)   the extrusion direction    [WCS]
+ * return   the transformed coordinate [OCS]
+ * created: 2012-02-03
+ */
+static V3Point crossProduct(const V3Point a, const V3Point b)
+{   V3Point result;
+
+    result.x = a.y * b.z - a.z * b.y;
+    result.y = a.z * b.x - a.x * b.z;
+    result.z = a.x * b.y - a.y * b.x;
+    return result;
+}
+static V3Point scaleToUnit(V3Point p)
+{   double  len = sqrt(p.x*p.x + p.y*p.y + p.z*p.z);
+
+    if (len != 0)
+    {   p.x /= len;
+        p.y /= len;
+        p.z /= len;
+    }
+    return p;
+}
+static V3Point transformOCS(V3Point pOCS, V3Point AZ)
+{   V3Point AX, AY, p;
+
+    if ( Abs(AZ.x) < 1.0/64.0 && Abs(AZ.y) < 1.0/64.0 )
+        AX = crossProduct(V3MakePoint(0.0, 1.0, 0.0), AZ);   // transform using unit vector Y
+    else
+        AX = crossProduct(V3MakePoint(0.0, 0.0, 1.0), AZ);   // transform using unit vector Z
+    AX = scaleToUnit( AX );  // scale vector to unit (length = 1.0)
+    AY = crossProduct(AZ, AX);
+    AY = scaleToUnit( AY );
+
+    /* these are direction cosines, we have to rotate each axis around this */
+    {   double x = pOCS.x, y = pOCS.y, z = pOCS.z;
+
+        p.x = x * AX.x + y * AY.x + z * AZ.x;
+        p.y = x * AX.y + y * AY.y + z * AZ.y;
+        p.z = x * AX.z + y * AY.z + z * AZ.z;
+    }
+    return p;
+}
+
 /* get group entries
  * scanner starts at id line of first entry
  * scanner scans until it reaches the next group
- * created:  19.03.97
+ * created:  1997-03-19
+ * modified: 2011-04-04 (group.z0 - z3 added)
+ *           2009-02-06 (read in extrusion direction)
  */
 - (BOOL)scanGroup:(NSScanner*)scanner
-{
+{   BOOL    exitLoop = NO;
+
     [group.text release]; group.text = nil;
     [group.name release]; group.name = nil;
     [group.layer release]; group.layer = nil;
@@ -1381,12 +1577,13 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
     group.width = -1.0;
     group.endWidth = -1.0;
     group.begAngle = group.endAngle = 0.0;
-    group.x0 = group.y0 = 0.0;
-    group.x1 = group.y1 = 0.0;
-    group.x2 = group.y2 = 0.0;
-    group.x3 = group.y3 = 0.0;
+    group.x0 = group.y0 = group.z0 = 0.0;
+    group.x1 = group.y1 = group.z1 = 0.0;
+    group.x2 = group.y2 = group.z2 = 0.0;
+    group.x3 = group.y3 = group.z3 = 0.0;
     group.a = 0.0;
     group.flags = group.genFlags = 0;
+    group.extX = group.extY = 0.0; group.extZ = 1.0;
 
     while ( ![scanner isAtEnd] )
     {	int	i, location = [scanner scanLocation];
@@ -1438,6 +1635,10 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
                 if (![scanner scanFloat:&group.y0])
                     group.y0 = 0.0;
                 break;
+            case ID_Z0:
+                if (![scanner scanFloat:&group.z0])
+                    group.z0 = 0.0;
+                break;
             case ID_X1:
                 if (![scanner scanFloat:&group.x1])
                     group.x1 = 0.0;
@@ -1446,6 +1647,10 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
                 if (![scanner scanFloat:&group.y1])
                     group.y1 = 0.0;
                 break;
+            case ID_Z1:
+                if (![scanner scanFloat:&group.z1])
+                    group.z1 = 0.0;
+                break;
             case ID_X2:
                 if (![scanner scanFloat:&group.x2])
                     group.x2 = 0.0;
@@ -1454,6 +1659,10 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
                 if (![scanner scanFloat:&group.y2])
                     group.y2 = 0.0;
                 break;
+            case ID_Z2:
+                if (![scanner scanFloat:&group.z2])
+                    group.z2 = 0.0;
+                break;
             case ID_X3:
                 if (![scanner scanFloat:&group.x3])
                     group.x3 = 0.0;
@@ -1462,6 +1671,10 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
                 if (![scanner scanFloat:&group.y3])
                     group.y3 = 0.0;
                 break;
+            case ID_Z3:
+                if (![scanner scanFloat:&group.z3])
+                    group.z3 = 0.0;
+                break;
             case ID_WIDTH:	/* begWidth, radius, sumLen, height */
                 if (![scanner scanFloat:&group.width])
                     group.width = -1.0;
@@ -1508,14 +1721,46 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
                 if (![scanner scanInt:&group.numGrp])
                     group.numGrp = 0;
                 break;
-           case ID_GROUP:
+            case ID_EXT_X:
+                if (![scanner scanFloat:&group.extX])
+                    group.extX = 0.0;
+                break;
+            case ID_EXT_Y:
+                if (![scanner scanFloat:&group.extY])
+                    group.extY = 0.0;
+                if ( group.extX != 0.0 || group.extY != 0.0 )   // FIXME: extrusion direction
+                    NSLog(@"DXF-Import: extrusion direction not supported. File-Location = %d", [scanner scanLocation]);
+                break;
+            case ID_EXT_Z:
+                if (![scanner scanFloat:&group.extZ])
+                    group.extZ = 1.0;
+                break;
+            case ID_GROUP:
                 [scanner setScanLocation:location];
-                return YES;
+                exitLoop = YES;
+                break;
             default:
                 break;
         }
+        if ( exitLoop )
+            break;
         gotoNextLine(scanner);	// goto id line
     }
+
+    /* transform coordinates from OCS to WCS (only if extrusion direction tells us so) */
+    if ( group.extX != 0.0 || group.extY != 0.0 || group.extZ != 1.0 )
+    {   V3Point N = V3MakePoint(group.extX, group.extY, group.extZ), p;
+
+        p = transformOCS(V3MakePoint(group.x0, group.y0, group.z0), N);
+        group.x0 = p.x; group.y0 = p.y; group.z0 = p.z;
+        p = transformOCS(V3MakePoint(group.x1, group.y1, group.z1), N);
+        group.x1 = p.x; group.y1 = p.y; group.z1 = p.z;
+        p = transformOCS(V3MakePoint(group.x2, group.y2, group.z2), N);
+        group.x2 = p.x; group.y2 = p.y; group.z2 = p.z;
+        p = transformOCS(V3MakePoint(group.x3, group.y3, group.z3), N);
+        group.x3 = p.x; group.y3 = p.y; group.z3 = p.z;
+    }
+
     return YES;
 }
 
@@ -1542,10 +1787,18 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
 {
     NSLog(@"line: %f %f %f %f", beg.x, beg.y, end.x, end.y); 
 }
+- (void)addLine3D:(V3Point)beg :(V3Point)end toList:(NSMutableArray*)aList
+{
+    NSLog(@"line 3D: %f %f %f %f %f %f", beg.x, beg.y, beg.z, end.x, end.y, end.z);
+}
 - (void)addLine:(NSPoint)beg :(NSPoint)end toLayer:(NSString*)layerName
 {
     NSLog(@"line: %@ %f %f %f %f", layerName, beg.x, beg.y, end.x, end.y); 
 }
+- (void)addLine3D:(V3Point)beg :(V3Point)end toLayer:(NSString*)layerName
+{
+    NSLog(@"line 3D: %@ %f %f %f %f %f %F", layerName, beg.x, beg.y, beg.z, end.x, end.y, end.z); 
+}
 
 - (void)addArc:(NSPoint)center :(NSPoint)start :(float)angle toList:(NSMutableArray*)aList
 {
@@ -1564,6 +1817,11 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
 {
     NSLog(@"curve: %@ %f %f %f %f %f %f %f %f", layerName, p0.x, p0.y, p1.x, p1.y, p2.x, p2.y, p3.x, p3.y);
 }
+- (void)add3DFace:(V3Point*)pts toLayer:(NSString*)layerName
+{
+    NSLog(@"3DFace: %@ %f %f %f, %f %f %f, %f %f %f, %f %f %f", layerName, pts[0].x, pts[0].y, pts[0].z,
+          pts[1].x, pts[1].y, pts[1].z, pts[2].x, pts[2].y, pts[2].z, pts[3].x, pts[3].y, pts[3].z);
+}
 
 /* allocate a text object and add it to layer
  * parameter: text	the text string
